#ifdef GL_ES
	precision mediump float;
#endif

#ifdef Caustic
	#define Point1			vec3(-12.897254, 4.869618, 30.252148)*0.1
	#define Point2			vec3(20.111871, 4.867618, 30.252148)*0.1
	#define Point3			vec3(-12.897254, 23.4596676, 18.178324)*0.1
	#define WidthVector 	(Point2-Point1)
	#define HeightVector	(Point3-Point1)
	#define DotWidth		dot(WidthVector, WidthVector)
	#define DotHeight		dot(HeightVector, HeightVector)
#endif

attribute vec3 a_position;
attribute vec3 a_normal;
attribute vec2 a_texCoord0;

#ifdef UV2
	attribute vec2 a_texCoord1;
#endif

#ifdef VertexColor
	attribute vec4 a_color;
#endif

#ifdef boneWeight0Flag
	#define boneWeightsFlag
	attribute vec2 a_boneWeight0;
#endif //boneWeight0Flag

#ifdef boneWeight1Flag
	#ifndef boneWeightsFlag
		#define boneWeightsFlag
	#endif
	attribute vec2 a_boneWeight1;
#endif //boneWeight1Flag

#ifdef boneWeight2Flag
	#ifndef boneWeightsFlag
		#define boneWeightsFlag
	#endif
	attribute vec2 a_boneWeight2;
#endif //boneWeight2Flag

#ifdef boneWeight3Flag
	#ifndef boneWeightsFlag
		#define boneWeightsFlag
	#endif
	attribute vec2 a_boneWeight3;
#endif //boneWeight3Flag

#ifdef boneWeight4Flag
	#ifndef boneWeightsFlag
		#define boneWeightsFlag
	#endif
	attribute vec2 a_boneWeight4;
#endif //boneWeight4Flag

#ifdef boneWeight5Flag
	#ifndef boneWeightsFlag
		#define boneWeightsFlag
	#endif
	attribute vec2 a_boneWeight5;
#endif //boneWeight5Flag

#ifdef boneWeight6Flag
	#ifndef boneWeightsFlag
		#define boneWeightsFlag
	#endif
	attribute vec2 a_boneWeight6;
#endif //boneWeight6Flag

#ifdef boneWeight7Flag
	#ifndef boneWeightsFlag
		#define boneWeightsFlag
	#endif
	attribute vec2 a_boneWeight7;
#endif //boneWeight7Flag

uniform mat4	u_projTrans;
uniform mat4	u_worldTrans;
uniform vec4	u_cameraPosition;
uniform vec3	u_lightDirection;
uniform float	u_ambientLight;

#ifdef normalMatrixFlag
	uniform mat3 u_normalMatrix;
#endif

#ifdef Light2
	uniform vec3	u_lightDirection2;
#endif

#ifdef Caustic
	uniform vec2	u_causticShift00;
	uniform vec2	u_causticShift01;
#endif

#ifdef Transparent
	uniform float	u_fogFar;
	uniform float	u_fogRange;
#endif

#if defined(numBones)
	uniform mat4 u_bones[numBones];
#endif

varying vec2	v_texCoord0;
varying float	v_NdotL;
varying vec3	v_lightDiffuse;

#ifdef Caustic
	varying vec2	v_causticCoord0;
	varying vec2	v_causticCoord1;
#endif

#ifdef UV2
	varying vec2	v_texCoord1;
#endif

#ifdef VertexColor
	varying vec3 v_color;
#endif

#if defined(Transparent) || defined(Fog)
	varying float	v_fog;
#endif

#if defined(numBones) && defined(boneWeightsFlag)
	#if (numBones > 0) 
		#define skinningFlag
	#endif
#endif

void main() {
	#ifdef skinningFlag
		mat4 skinning = mat4(0.0);
		#ifdef boneWeight0Flag
			skinning += (a_boneWeight0.y) * u_bones[int(a_boneWeight0.x)];
		#endif //boneWeight0Flag
		#ifdef boneWeight1Flag				
			skinning += (a_boneWeight1.y) * u_bones[int(a_boneWeight1.x)];
		#endif //boneWeight1Flag
		#ifdef boneWeight2Flag		
			skinning += (a_boneWeight2.y) * u_bones[int(a_boneWeight2.x)];
		#endif //boneWeight2Flag
		#ifdef boneWeight3Flag
			skinning += (a_boneWeight3.y) * u_bones[int(a_boneWeight3.x)];
		#endif //boneWeight3Flag
		#ifdef boneWeight4Flag
			skinning += (a_boneWeight4.y) * u_bones[int(a_boneWeight4.x)];
		#endif //boneWeight4Flag
		#ifdef boneWeight5Flag
			skinning += (a_boneWeight5.y) * u_bones[int(a_boneWeight5.x)];
		#endif //boneWeight5Flag
		#ifdef boneWeight6Flag
			skinning += (a_boneWeight6.y) * u_bones[int(a_boneWeight6.x)];
		#endif //boneWeight6Flag
		#ifdef boneWeight7Flag
			skinning += (a_boneWeight7.y) * u_bones[int(a_boneWeight7.x)];
		#endif //boneWeight7Flag
		
		vec4 pos	= u_worldTrans * skinning * vec4(a_position, 1.0);
   		vec3 normal	= normalize((u_worldTrans * skinning * vec4(a_normal, 0.0)).xyz);
	#else
		vec4 pos	= u_worldTrans * vec4(a_position, 1.0);
		
		#ifdef normalMatrixFlag
			vec3 normal = normalize(u_normalMatrix * a_normal);
		#else
   			vec3 normal	= normalize(a_normal);
		#endif
	#endif //skinningFlag
	
	gl_Position =  u_projTrans * pos;
	v_texCoord0 = a_texCoord0;
	
	#ifdef UV2
		v_texCoord1 = a_texCoord1;
	#endif

	#ifdef VertexColor
		v_color = a_color.rgb;
	#endif
	
	// =============================
	// Light Diffuse
	// =============================
	v_NdotL			= max(dot(normal, u_lightDirection)*1.3, 0.0);
    v_lightDiffuse	= vec3(u_ambientLight);
	v_lightDiffuse += vec3(0.4, 0.7, 1.0) * v_NdotL;
    
#ifdef Light2
    v_lightDiffuse += vec3(0.4, 0.7, 1.0) * max(dot(normal, u_lightDirection2), 0.0);
#endif
	
#ifdef Caustic
	vec3 causticVector = pos.xyz-Point1;
	float causticX = dot(causticVector, WidthVector)/DotWidth;
	float causticY = dot(causticVector, HeightVector)/DotHeight;
	v_causticCoord0 = vec2(causticX, causticY)/0.8-u_causticShift00;
	v_causticCoord1 = vec2(causticX, causticY)-u_causticShift01;
#endif
	
#ifdef Fog
    vec3 flen = u_cameraPosition.xyz - pos.xyz;
    float fog = min(dot(flen, flen), 50.0) * u_cameraPosition.w; // min(dot(flen, flen), 39.0) * u_cameraPosition.w;
    v_fog = min(fog, 1.0);
#endif
	
#ifdef Transparent
    vec3 flen = u_cameraPosition.xyz - pos.xyz;
    float fog = (u_fogFar-dot(flen, flen))/u_fogRange; // min(dot(flen, flen), 39.0) * u_cameraPosition.w;
    v_fog = min(fog, 1.0);
#endif
}
