#ifdef GL_ES
	precision mediump float;
#endif

attribute vec3	a_position;
attribute vec2	a_texCoord0;

uniform mat4	u_projTrans;
uniform mat4	u_worldTrans;

varying vec2	v_texCoord0;

#ifdef Beam
	uniform vec2 u_innerTime;
	
	varying vec2	v_beamCoord0;
#endif

#ifdef Bubble
	uniform float	u_frameTime;
	
	varying vec2	v_bubbleCoord;
#endif

#ifdef Common
	uniform vec4	u_cameraPosition;
	
	varying float	v_fog;
#endif

#ifdef Ocean
	attribute vec3	a_normal;
	
	uniform vec3	u_lightPoint;
	uniform vec2	u_oceanShift00;
	uniform vec2	u_oceanShift01;
	uniform float	u_frameTime;
	
	varying vec4	v_point;
	varying vec3	v_lightDiffuse;
	varying vec2	v_waveCoord0;
	varying vec2	v_waveCoord1;
	varying float	v_NdotL2;
#endif

const float one = 1.0;
const float two = 2.0;
const vec3 v3one = vec3(1.0);

void main() {
	vec4 pos	= u_worldTrans * vec4(a_position, one);
	gl_Position = u_projTrans * pos;
	
	v_texCoord0 = a_texCoord0;
	
	#ifdef Beam
		v_beamCoord0 = a_texCoord0 - vec2(0.5, 0.5);
		v_beamCoord0 = vec2(v_beamCoord0.x * u_innerTime.x + v_beamCoord0.y * u_innerTime.y, v_beamCoord0.y * u_innerTime.x - v_beamCoord0.x * u_innerTime.y);
		v_beamCoord0+= vec2(0.5, 0.5);
	#endif
	
	#ifdef Bubble
		v_bubbleCoord = a_texCoord0 + vec2(0.0, u_frameTime);
	#endif
	
	#ifdef Common
	    vec3 flen	= vec3(pos.x, 0.0, pos.z);
	    v_fog		= min(dot(flen, flen) * u_cameraPosition.w, one);
	#endif

	#ifdef Ocean
		v_point		= pos;
		// UӻUǪ@γOFCBq
		vec2 deltaTex = a_texCoord0 * 0.1;
		
		v_waveCoord0 = a_texCoord0 + deltaTex - u_oceanShift00;
		v_waveCoord1 = a_texCoord0 - deltaTex - u_oceanShift01;
		
		vec3 lightVector	= normalize(u_lightPoint - pos.xyz);
		float NdotL			= dot(a_normal, lightVector);
		v_NdotL2			= NdotL * NdotL;
		v_lightDiffuse		= v3one + NdotL;
	#endif
}