precision mediump float;

varying vec4	v_Position;
varying vec2	v_TextureCoord;
varying float	v_FogDepth;

uniform sampler2D	u_BaseTexture;
uniform sampler2D	u_WaveTexture;
uniform vec3		u_CameraPosition;
uniform float		u_Time;
uniform float		u_FogNear;
uniform float		u_FogFar;

const float eta		= 0.3;
const float maxTime	= 65535.0;

void main()
{
	vec2 textureCoord;
	vec3 normalVector, eyeVector, refractVector, lightDiffuse, lightVector, lightPoint = vec3(0.0, 0.0, 0.6);
	
	normalVector  = texture2D(u_WaveTexture, v_TextureCoord+u_Time/vec2(maxTime, maxTime)).xzy - vec3(0.5);
	textureCoord  = vec2(v_TextureCoord.x-v_TextureCoord.y, v_TextureCoord.x+v_TextureCoord.y);
	normalVector += texture2D(u_WaveTexture, textureCoord+u_Time/vec2(maxTime, -maxTime)).xzy - vec3(0.5);
	normalVector += texture2D(u_WaveTexture, v_TextureCoord+u_Time/vec2(maxTime, -0.5*maxTime)).xzy - vec3(0.5);
	normalVector = normalize(normalVector);
	
	float fogDensity = (v_FogDepth - u_FogNear) / (u_FogFar - u_FogNear);
	
	eyeVector		= normalize(u_CameraPosition-v_Position.xyz);
	refractVector	= refract(eyeVector, normalVector, eta);
	float m = 2.0*sqrt(refractVector.x*refractVector.x+refractVector.y*refractVector.y+(refractVector.z+1.0)*(refractVector.z+1.0));
	float s = refractVector.x/m+0.5;
	float t = refractVector.y/m+0.5;
	//m = 2.0*sqrt(t.x*t.x + t.y*t.y + (t.z+1.0))*(t.z +1.0);
	//s = t.x/m+0.5
	//t = t.y/m+0.5
	vec4 color		= texture2D(u_BaseTexture, vec2(s, t));
	
	refractVector	= refractVector * (0.5/sqrt(dot(refractVector, refractVector))) + 0.5;
	//vec4 color		= texture2D(u_BaseTexture, vec2(refractVector.x, refractVector.y));
	
	float NdotL; 
	lightVector	= normalize(lightPoint - v_Position.xyz);
	NdotL		= max( dot(vec3(0.0, 0.0, 1.0), lightVector), 0.0);
	lightDiffuse= vec3(1.0) + NdotL*NdotL*NdotL*2.3;
	
	vec3 testeye = normalize(vec3(0.0, 0.0, 5.0)-v_Position.xyz);
	vec3 test = normalize(vec3(v_Position.xy, 1.0));
	float try = pow(1.0-dot(test, testeye), 2.0);
	
	float fresnel = pow(1.0-dot(normalVector, eyeVector), 2.0);// * try * try * 0.8;
	
	//gl_FragColor = vec4(color.rgb*lightDiffuse, (1.0 - fresnel) * (1.0 - fogDensity) * 1.5 + NdotL*NdotL);
	gl_FragColor = vec4(color.rgb, (1.0-fresnel) * (1.0 - fogDensity) * 1.5);
}

/*
vec3 normal(vec2 coord, vec2 time) {return texture2D(u_WaveTexture, coord+u_Time/time).xzy - vec3(0.5);}

	N += normal(v_TextureCoord, vec2(MAX_T, MAX_T * 1.0));
	N += normal(v_TextureCoord, vec2(-MAX_T, -MAX_T * 0.5));
	
	vec2 texCoord2 = vec2(v_TextureCoord.x + v_TextureCoord.y, v_TextureCoord.y - v_TextureCoord.x);
	N += normal(texCoord2, vec2(MAX_T * 1.0, -MAX_T * 1.0));
	N = normalize(N);
	
	float fogDensity = (v_FogDepth - u_FogNear) / (u_FogFar - u_FogNear);
	vec3 I = normalize(u_CameraPosition-v_Position.xyz);
	vec3 T = refract(I, normalVector, ETA);
	float m = 2.0 * sqrt(T.x * T.x + T.y * T.y + (T.z + 1.0) * (T.z+1.0));
	float s = T.x/m + 0.5;
	float t = T.y/m + 0.5;
	vec4 color = texture2D(u_BaseTexture, vec2(s, t));
	vec3 V = normalize(v_Position.xyz - u_CameraPosition);
	float fresnel = pow(1.0 - dot(normalVector, V), 2.0)    * 0.85;
	
	gl_FragColor = vec4(color.rgb, (1.0 - fresnel) * (1.0 - fogDensity) * 1.5);
*/